% a seqential Monte Carlo version of simulated annealing
% maximization problem

% Authors: 
% Xi Chen (xchen37@illinois.edu) 
% Enlu Zhou (enlu.zhou@isye.gatech.edu)


function [samples, Hbar, xbar] = SMCSA(N, K, choices, M)
% N: # of particles
% K: # of iterations
% choices: choice of test function
% M: dimension of the problem
% samples: number of samples
% Hbar: Optimal value
% xbar: Candidate solution

% intialization
% X is a N*M matrix, each row is a sample
X = rand(N,M).*100-50;
T = 5*10^6;    % initial temperature
%gamma = 0.95;    % temperature reduction factor
Hbar(1) = -inf;
samples(1)=0;
alpha=10;
beta=0.998;

for i = 2:1:K
    samples(i) = (i-1)*N;
    % change temperature
    %T1 = T*gamma+1e-50;
    T1=10^(-10)+10^(-1)*abs(Hbar(i-1))/log(i+1);
    % evaluate weights
    switch choices
        case 1
            G=rosenbrock(X);
        case 2
            G =tri(X);
        case 3
            G=griewank(X);
        case 4
            G=dejong5(X);
        case 5
            G=powel(X);
        case 6
            G = pinter(X);
    end
    % multiply 10^(-3) to prevent degenerate weight
    W = exp(G.*10^(-3)*(1/T1-1/T)) + 1e-30; % used in griewank, rosenbrock, tri, powel, pinter
    %W = exp(G.*(1/T1-1/T)) + 1e-30; % used in dejong5
    W = W./sum(W);

    % estimate max H and x in this interation
    [Hbar(i), indx] = max(G);
    xbar(i,:) = X(indx,:);
     
    % resample
    Xresamp = stratified_resample(X,W,N);
    
    % SA move
    Y = Xresamp + randn(N,M)*(alpha*beta^i);
    % acceptance prob
    switch choices
        case 1
            dG=rosenbrock(Y)-rosenbrock(Xresamp);
        case 2
            dG = tri(Y)-tri(Xresamp);
        case 3
            dG= griewank(Y)-griewank(Xresamp);
        case 4
            dG = dejong5(Y)-dejong5(Xresamp);
        case 5
            dG = powel(Y)-powel(Xresamp);
        case 6
            dG = pinter(Y)-pinter(Xresamp);
    end
    rho = min(exp(dG./T1),1);
    % accept & reject
    U = rand(N,1);
    X = Y.*((U <= rho)*ones(1,M)) + Xresamp.*((U>rho)*ones(1,M));
    T=T1;
    clear Y; clear Xresamp; clear G; clear W; clear dG;
end
    
% figure;
% semilogy(samples, Hbar);